/*
 *  Copyright (C) 2008 dhewg, #wiidev efnet
 *
 *  this file is part of geckoloader
 *  http://wiibrew.org/index.php?title=Geckoloader
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <gccore.h>
#include <ogcsys.h>
#include <debug.h>

#include "asm.h"
#include "processor.h"

#include "gecko.h"
#include "elf.h"
#include "dol.h"
#include "../../config.h"

static void *xfb = NULL;
static GXRModeObj *rmode;

extern void __exception_closeall();

typedef void (*entrypoint) (void);

// TODO
// add reset/power cb, but i need a way to abort the blocking transfer 1st

int main() {
        void *data;
        s32 res;
        u32 entry;
        entrypoint entry_p;

        AUDIO_Init (NULL);
        DSP_Init ();
        AUDIO_StopDMA();
        AUDIO_RegisterDMACallback(NULL);

        VIDEO_Init ();

		rmode = VIDEO_GetPreferredMode (NULL);

        xfb = MEM_K0_TO_K1 (SYS_AllocateFramebuffer (rmode));

        VIDEO_Configure (rmode);
        VIDEO_SetNextFramebuffer (xfb);
        VIDEO_SetBlack (FALSE);
        VIDEO_Flush ();
        VIDEO_WaitVSync ();
        if (rmode->viTVMode & VI_NON_INTERLACE)
                VIDEO_WaitVSync ();

        CON_InitEx (rmode, 20, 30, rmode->fbWidth - 40, rmode->xfbHeight - 60);

        printf ("geckoloader loading code " GECKOLOADER_VERSION
#ifdef DEBUG_GECKOLOADER
                " DEBUG version\n"
#else
                "\n"
#endif
                "coded by dhewg, #wiidev efnet/blitzed\n");

        while (1) {
                printf ("\nready to receive a gamecube binary...\n");

                data = gecko_receive ();

                if (!data) {
                        printf ("error receiving the binary\n\n");
                        continue;
                }

                entry = 0;
                res = valid_elf_image (data);

                if (res < 0) {
                        printf ("this is not a valid 32bit ppc .elf\n");
                        continue;
                }

                if (res == 1) {
                        printf ("detected a .elf file\n");
                        entry = load_elf_image (data);
                } else {
                        printf ("seems to be a .dol file\n");
                        entry = load_dol_image (data);
                }

                if (entry == 0) {
                        printf ("error: there is no entry point to jump to\n");
                        continue;
                }

                printf ("jumping to 0x%08x\n", entry);
                VIDEO_WaitVSync();

                entry_p = (entrypoint) entry;

				SYS_ResetSystem (SYS_SHUTDOWN, 0, 0);
                __exception_closeall ();
                entry_p ();

                printf ("eh?\n");
                VIDEO_WaitVSync ();
        }

        return 0;
}

